/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

gc.autoDetectPortSelected = function(transport, port, info)
{
    var controllerName = transport.getControllerName();
    if (port)
    {
        if (controllerName) 
        {
			console.log('Controller ' + controllerName + ' is using port: ' + port.comName);
        }
		
	  
		var portIdentity = info.getAutoDetectIdentity(port);
		var reg_model = document.querySelector('#my_model');
		if (portIdentity && portIdentity === "usb2any")
		{
		    transport.usb = false;
            transport.hid = true;
            transport.type="usb2any"
			reg_model.protocol = "usb2any";
		}
		
		if (portIdentity && portIdentity === "aevm")
		{
		    transport.usb = true;
            transport.hid = false;
            transport.type="aevm"
			reg_model.protocol = "aevm";
		}
	    
    }
};

gc.autoDetectPortIdentity = function(port, helperAPI)
{

   if (port.productId && port.productId === 769)
   {
       return "usb2any";
   }
   if (port.pnpId)
   {
       const match_expr = /PID_2014/g;
       if (port.pnpId.search(match_expr) > 0)
       {
           return "aevm";
       }
   }
    
    
};

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/

	gc.databind.registry.getBinding('my_model.PID').addChangedListener({
        onValueChanged(oldValue, newValue) {
            Polymer.dom(document).querySelector('ti-model-register').getRegisterViewInformation().then(function(deviceObj) {
                // filter the register blocks based on id
                const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.name === "UserRegisters-user_mem"));
                if (regBlockArray.length > 0) {
                    const regBlock = regBlockArray[0];
                    const safetyRegs = regBlock.registers.filter((reg) => (reg.addr > 0x2A && reg.addr < 0x36)); 
                    const watchdogRegs = regBlock.registers.filter((reg) => (reg.addr > 0x36 && reg.addr < 0x4A)); 
                    const monRegs = regBlock.registers.filter((reg) => (reg.addr > 0x22 && reg.addr < 0x27));
                    const intRegs= regBlock.registers.filter((reg) => (reg.addr > 0x18 && reg.addr < 0x1E));
                    
                    const GPIOControlReg = regBlock.registers.filter((reg) => (reg.name === "GPIO_CTRL"));
                    const GPIOnINTConfigReg = regBlock.registers.filter((reg) => (reg.name === "LDO_GPIO_CFG"));
                    const GPIOSeqRegs = regBlock.registers.filter((reg) => reg.name === "SEQ_TRIG_GPIO" || reg.name === "GPIO_SEQ_DLY");
                    const intMaskBuckxLDOReg = regBlock.registers.filter((reg) => reg.name === "INT_MASK_BUCKx_LDO");
                    const pinMonErrCfgReg = regBlock.registers.filter((reg) => reg.name === "PIN_MON_ERR_CFG");
                    const errBuckxLDOReg = regBlock.registers.filter((reg) => reg.name === "ERR_BUCKx_LDO_CFG");
                    
                    // hide watchdog regs
                    for (var i = 0; i < watchdogRegs.length; i++) {
                        watchdogRegs[i].attrs = watchdogRegs[i].attrs || {};
                        watchdogRegs[i].attrs.isHidden = (newValue - 0xE0 < 0) ? "1" : "0";
                    }
                    
                    // hide other safety regs
                    for (i = 0; i < safetyRegs.length; i++) {
                        safetyRegs[i].attrs = safetyRegs[i].attrs || {}; 
                        safetyRegs[i].attrs.isHidden = (newValue - 0xC0 < 0) ? "1" : "0";
                    }

                    // hide int regs
                    for (i = 0; i < intRegs.length; i++) {
                        intRegs[i].attrs = intRegs[i] || {};
                        intRegs[i].attrs.isHidden = (newValue - 0x90 < 0) ? "1" : "0";
                    }
                    
                    // hide ov monitors 
                    for (i = 0; i < monRegs.length; i++) {
                        monRegs[i].fields[1].attrs.isHidden = (newValue - 0xC0 < 0) ? "1" : "0";
                        monRegs[i].fields[1].mode = (newValue - 0xC0 < 0) ? "R" : "RW";
                    }
                    
                    for (i = 0; i < 8; i++) {
                        if (i % 2 === 1) {
                            errBuckxLDOReg[0].fields[i].attrs.isHidden = (newValue - 0xC0 < 0) ? "1" : "0";
                            errBuckxLDOReg[0].fields[i].mode = (newValue - 0xC0 < 0) ? "R" : "RW";
                        }
                    }
                    
                    hideGPIOSEQ = gc.databind.registry.getBinding('widget.my_model.LDO_GPIO_CFG.GPIO_DIR_CONFIG').getValue() || newValue - 0x80 < 0;
                    templateObj.$.ti_widget_container_SEQ_GPIO.nonvisible = hideGPIOSEQ; 
                    
                    // hide ov led indicators 
                    const hideOV = (newValue - 0xC0 < 0) ? 1 : 0
                    
                    templateObj.$.B1_OV_LED.hidden = hideOV;
                    templateObj.$.B2_OV_LED.hidden = hideOV;
                    templateObj.$.B3_OV_LED.hidden = hideOV;
                    templateObj.$.LDO_OV_LED.hidden = hideOV;
                    templateObj.$.B1_OV_INT_LED.hidden = hideOV;
                    templateObj.$.B2_OV_INT_LED.hidden = hideOV;
                    templateObj.$.B3_OV_INT_LED.hidden = hideOV;
                    templateObj.$.LDO_OV_INT_LED.hidden = hideOV;
                    templateObj.$.BUCK1_OV_RST_EVENT.hidden = hideOV;
                    templateObj.$.BUCK2_OV_RST_EVENT.hidden = hideOV;
                    templateObj.$.BUCK3_OV_RST_EVENT.hidden = hideOV;
                    templateObj.$.LDO_OV_RST_EVENT.hidden = hideOV;
                    
                    
                    //hide safety registers from save settings output
                    const hideSafetyBindings = (newValue - 0xC0 < 0 ? {} : (function() {return;})());
                    
                    gc.databind.registry.getBinding('widget.my_model.DEV_ERR_ACK_1').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.DEV_ERR_ACK_2').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.DEV_FAULT_STATUS_1').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.DEV_FAULT_STATUS_2').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.INT_MASK_SAFETY').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.PIN_MON_ERR_CFG').excludeFromStorageProviderData = hideSafetyBindings;
                    //gc.databind.registry.getBinding('widget.my_model.POWER_GOOD_STATUS').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.ABIST_GROUP_DONE').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.ABIST_BUCK1_2_ACK').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.ABIST_LDO_BUCK3_ACK').excludeFromStorageProviderData = hideSafetyBindings;
                    gc.databind.registry.getBinding('widget.my_model.ABIST_SYSTEM_ACK').excludeFromStorageProviderData = hideSafetyBindings;  
                    gc.databind.registry.getBinding('widget.my_model.ABIST_RUN_CMD').excludeFromStorageProviderData = hideSafetyBindings;
                    
                    //hide watchdog registers from save settings output
                    const hideWatchdogBindings = (newValue - 0xE0 < 0 ? {} : (function() {return;})());
                    
                    gc.databind.registry.getBinding('widget.my_model.WD_ANSWER_REG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_QUESTION_ANSW_CNT').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_WIN1_CFG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_WIN2_CFG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_LONGWIN_CFG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_MODE_REG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_QA_CFG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_ERR_STATUS').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_THR_CFG').excludeFromStorageProviderData = hideWatchdogBindings
                    gc.databind.registry.getBinding('widget.my_model.WD_FAIL_CNT_REG').excludeFromStorageProviderData = hideWatchdogBindings
                    
                    // hide GPIO/nINT settings for Fedora Light
                    const hideGPIOnINT = (newValue < 0x80 ? "1" : "0");
                    const GPIOnINTReadOnly = (newValue < 0x80 ? "R" : "RW");    
                        
                    // GPIO state field
                    GPIOControlReg[0].fields[0].attrs.isHidden = hideGPIOnINT;
                    GPIOControlReg[0].fields[0].mode = GPIOnINTReadOnly;
                    
                    // GPIO direction field
                    GPIOnINTConfigReg[0].fields[2].attrs.isHidden = hideGPIOnINT;
                    GPIOnINTConfigReg[0].fields[2].mode = GPIOnINTReadOnly;
                    
                    // nINT configuration field
                    GPIOnINTConfigReg[0].fields[3].attrs.isHidden = hideGPIOnINT; 
                    GPIOnINTConfigReg[0].fields[3].mode = GPIOnINTReadOnly;
                    
                    // nINT mask register
                    intMaskBuckxLDOReg[0].mode = GPIOnINTReadOnly;
                    
                    // nINT error mask
                    pinMonErrCfgReg[0].fields[1].attrs.isHidden = hideGPIOnINT; 
                    pinMonErrCfgReg[0].fields[1].mode = GPIOnINTReadOnly;
                    
                    // GPIO seq trigger and delay registers
                    for (i = 0; i < GPIOSeqRegs.length; i++) {
                        GPIOSeqRegs[i].mode = GPIOnINTReadOnly;
                    }
                    
                }
            });
        }
    });
    
//    gc.databind.registry.bind('widget.B1_UVLO_LED.on', "my_model.FAULT_STATUS_BUCK1.PVIN_UVLO_STATUS", function(value) { return value; }, function(value) { return value; });
//	gc.databind.registry.getBinding('widget.my_model.FAULT_STATUS_BUCK1.PVIN_UVLO_STATUS').addStreamingListener("click");

});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    regData = JSON.parse(contents);
                    if (regData) {
                        templateObj.setRegisterData(regData);
                        templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    }
    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: saved to " + result.localPath);
                    }

                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: saved to " + result.localPath);
                    }
                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8')
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_log_pane', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_eventlog_view.openView();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_scripting_dialog', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.scripting_dialog.removeAttribute('hidden');
            templateObj.$.scripting_dialog.open();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    
    

    

});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            console.log("Application template has been stamped.");
            templateObj.$.ti_widget_toast.hideToast();
            templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
            if (registerModel) {
                registerModel.addEventListener("usbHidModelReceivingData", function (event) {
                    templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false); // changed visible to true to test comm
                });
            } else {
                console.log("no ti-model-register component found in this application!")
            }

            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

            // open the eventlog viewer when the user clicks on the book icon in the status bar:
            templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
            templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function(event) {
                templateObj.$.ti_widget_eventlog_view.toggleView();
            });


            // Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
            templateObj.toggleMenu = function(event){
                templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
            };
            templateObj.$.ti_widget_icon_button_menu.addEventListener('click',templateObj.toggleMenu);
            templateObj.$.ti_widget_label_menu.addEventListener('click',templateObj.toggleMenu);

            if (templateObj.$.ti_widget_register_page) {
                // get the active device name from the register page
                templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

                // called when the user clicks the Write Register button:
                templateObj.$.ti_widget_register_page.addEventListener("write_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read Register button:
                templateObj.$.ti_widget_register_page.addEventListener("read_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user clicks the Read All Registers button:
                templateObj.$.ti_widget_register_page.addEventListener("read_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));

                // called when the user changes the immediate / deferred mode
                templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
                }.bind(this));

                // called when the user changes the auto read refresh rate
                templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
                }.bind(this));
            }

            var usbHidService = gc.services['ti-service-usbhid'];
            if (usbHidService) {
                usbHidService.addEventListener('usbHidError', function(event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
                })
            }

            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "oneui-register-data",
                    data: []
                }
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var i = 0; i < regBlocks.length; i++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[i].registers.length; r++) {
                        var reg = regBlocks[i].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        var item = {
                            idx: r,
                            id: regName,
                            value: reg.value
                        }
                        registersData.push(item);
                    }
                    regData.data.push(registersData);
                }
                var strJson = JSON.stringify(regData);
                return strJson;
            };

            templateObj.setRegisterData = function(strJson) {
                var regData = JSON.parse(strJson);
                var ok = true;
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var b = 0; b < regBlocks.length; b++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[b].registers.length; r++) {
                        var reg = regBlocks[b].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        item = regData.data[b][r];
                        if (item.id === regName) {
                            reg.value = item.value;
                            var intValue = gc.utils.getValueFromHexString(""+reg.value);
                            templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
                        } else {
                            var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                        }
                    }

                }
            };

            templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
                templateObj.$.ti_widget_toast.hideToast();
                templateObj.$.ti_widget_toast.message = msg;
                switch (msgType.toLowerCase()) {
                    case "error":
                        templateObj.$.ti_widget_toast.iconName = "error";
                        templateObj.$.ti_widget_toast.backgroundColor = "red";
                        break;
                    case "warning":
                        templateObj.$.ti_widget_toast.iconName = "warning";
                        templateObj.$.ti_widget_toast.backgroundColor = "orange";
                        break;
                    default:
                        templateObj.$.ti_widget_toast.iconName = "check";
                        templateObj.$.ti_widget_toast.backgroundColor = "teal";
                        break;
                }
                templateObj.$.ti_widget_toast.duration = +durationInSeconds;
                templateObj.$.ti_widget_toast.showToast();
                console.log("toast msg [" + msgType + "]: " + msg);
            };

            templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
                var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
                if (showInStatusBar) {
                    var statusBar = gc.services['ti-widget-statusbar'];
                    if (statusBar) {
                        statusBar.statusString3 = msg;
                        statusBar.setIcon(iconName);
                    }
                }
                if (showInToast) {
                    templateObj.showToastMsg(msgType, msg, 4);
                }
                templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
            };



			
//			ti_widget_image3
		//	templateObj.$.ti_widget_icon_button7.addEventListener("click",function(event){
			templateObj.$.ti_widget_image3.addEventListener('click',function(event){
				
				
//				var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
//				var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
//				var i2c = interfaces.i2c;

				/*
				var if_type = 3; // interface type for i2c
				var unit = 1;
				var command = 5;
				var params = [[0,0,0,0], [88, 0, 0,0], [80,0,0,0], [0,0,0,0], [1,0,0,0]];
				var payload = [0];*/
				
//				var write_value = templateObj.$.ti_widget_input1.value;
				

//				i2c.I2C_Write(1,0x60,[0x4A,0x2D]);

				var bind = gc.databind.registry.getBinding('widget.my_model.EEPROM_PROG_CMD');
				bind.setValue(0x2D);

				templateObj.logMsgAndNotify("check", "Device EEPROM Programmed", false, true);
//				i2c.write(0x07,0x1F);
				
/*				i2c.h2c_command(3, 1, 5, params)  // host to controller command
					.then(function(result) {
					console.log(result); 
						}).fail(function(err) {
					console.log(err); // handle error here
					
				});*/

            });
			
			
			
	    if (templateObj.$.ti_widget_button_connect) {
                templateObj.$.ti_widget_button_connect.addEventListener("click",function(event){
                    if (gc.connectionManager){
                        gc.connectionManager.connect();
                    }
                    
                    
                });
	    }
	    
//   CONTROL Button Functionality (Click automatically inverts '.checked' value)
             templateObj.$.LOCKED_CONTROL_REGISTERS.addEventListener("click",function(event){   
                
//                var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
//				var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
//				var i2c = interfaces.i2c;
				
				var bind = gc.databind.registry.getBinding('widget.my_model.CONTROL_LOCK'); 
				bind.setValue(0xDD);
				
//				bind.setValue(templateObj.$.LOCKED_CONTROL_REGISTERS.checked ? 0 : 0xDD);
				
				if(templateObj.$.LOCKED_CONTROL_REGISTERS.checked){
//				    i2c.I2C_Write(1,0x60,[0x02,0x00]); // Write Control lock instruction
//				    bind.setValue(0x00);
				    
				    templateObj.$.ti_widget_register_page.writeValueToTargetRegister(0, null, 0, 2);
				    
				    templateObj.logMsgAndNotify("check", "Control Registers Locked", false, true);
				} else {
//				    i2c.I2C_Write(1,0x60,[0x02,0xDD]); // Write Control unlock instruction
				    bind.setValue(0xDD);
				    templateObj.logMsgAndNotify("check", "Control Registers Unlocked", false, true);
				}
           });
           
//   CONFIG Button Functionality (Click automatically inverts '.checked' value)
             templateObj.$.LOCKED_CONFIG_REGISTERS.addEventListener("click",function(event){   
                
//              var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
//				var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
//				var i2c = interfaces.i2c;
				
				var bind = gc.databind.registry.getBinding('widget.my_model.CONFIG_LOCK');
				bind.setValue(0xAA);
				
				if(templateObj.$.LOCKED_CONFIG_REGISTERS.checked){
				//    i2c.I2C_Write(1,0x60,[0x05,0x00]); // Write Control lock instruction
				    bind.setValue(0x00);
				    templateObj.logMsgAndNotify("check", "Configuration Registers Locked", false, true);
				} else {
				//    i2c.I2C_Write(1,0x60,[0x05,0xAA]); // Write Control unlock instruction
				    bind.setValue(0xAA);
				    templateObj.logMsgAndNotify("check", "Configuration Registers Unlocked", false, true);
				}
           });
           
//   Reset PMIC Button
             templateObj.$.Reset_Button.addEventListener("click",function(event){   
                
                var bind = gc.databind.registry.getBinding('widget.my_model.RESTART_CMD');
                
                bind.setValue(0xDD); 
                
                /*var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
				var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
				var i2c = interfaces.i2c;
				
				    i2c.I2C_Write(1,0x60,[0x28,0xDD]); // Writes reset instruction to PMIC*/
				    templateObj.logMsgAndNotify("check", "PMIC Reset Instruction Sent", false, true);
				
           });
           
//   Clear Buck1 Interrupts Button
             templateObj.$.B1_INT_CLEAR_BTN.addEventListener("click",function(event){   

			var bind = gc.databind.registry.getBinding('widget.my_model.INT_ACK_BUCK1');  
			bind.setValue(0xFF);
                
//                var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
//				var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
//				var i2c = interfaces.i2c;

//              i2c.I2C_Write(1,0x60,[0x1A,0xFF]); // Write all 1s to Buck1 INT_ACK register
				    templateObj.logMsgAndNotify("check", "Buck1 Interrupt Clear Sent", false, true);
				    
				    
				    
				
           });
//   Clear Buck1 Faults Button
             templateObj.$.B1_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
            //    var mycodec  = templateObj.$.my_model.getCodec(); // get codec from register model
			//	var interfaces = mycodec.getInterfaces(); // get interfaces of gpio, i2c, etc.
			//	var i2c = interfaces.i2c;
			    var bind = gc.databind.registry.getBinding('widget.my_model.FAULT_STATUS_BUCK1');  
			    bind.setValue(0xFF);
			
			//	    i2c.I2C_Write(1,0x60,[0x1F,0xFF]); // Write all 1s to Buck1 FAULT_STATUS register
			   templateObj.logMsgAndNotify("check", "Buck1 Fault Clear Sent", false, true);
			
  	
           
                 
                 
             });
            
//   Clear Buck2 Interrupts Button
             templateObj.$.B2_INT_CLEAR_BTN.addEventListener("click",function(event){   

			    var bind = gc.databind.registry.getBinding('widget.my_model.INT_ACK_BUCK2');  
			    bind.setValue(0xFF);
			    
//				    i2c.I2C_Write(1,0x60,[0x1B,0xFF]); // Write all 1s to Buck2 INT_ACK register
				    templateObj.logMsgAndNotify("check", "Buck2 Interrupt Clear Sent", false, true);
				
           });
//   Clear Buck2 Faults Button
             templateObj.$.B2_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
                var bind = gc.databind.registry.getBinding('widget.my_model.FAULT_STATUS_BUCK2');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x20,0xFF]); // Write all 1s to Buck2 FAULT_STATUS register
				    templateObj.logMsgAndNotify("check", "Buck2 Fault Clear Sent", false, true);
				
           });
           
//   Clear Buck3 Interrupts Button
             templateObj.$.B3_INT_CLEAR_BTN.addEventListener("click",function(event){   
                
                var bind = gc.databind.registry.getBinding('widget.my_model.INT_ACK_BUCK3');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x1C,0xFF]); // Write all 1s to Buck3 INT_ACK register
				    templateObj.logMsgAndNotify("check", "Buck3 Interrupt Clear Sent", false, true);
				
           });
//   Clear Buck3 Faults Button
             templateObj.$.B3_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.FAULT_STATUS_BUCK3');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x21,0xFF]); // Write all 1s to Buck3 FAULT_STATUS register
				    templateObj.logMsgAndNotify("check", "Buck3 Fault Clear Sent", false, true);
				
           });
           
//   Clear LDO Interrupts Button
             templateObj.$.LDO_INT_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.INT_ACK_LDO');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x1D,0xFF]); // Write all 1s to LDO INT_ACK register
				    templateObj.logMsgAndNotify("check", "LDO Interrupt Clear Sent", false, true);
				
           });
//   Clear LDO Faults Button
             templateObj.$.LDO_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.FAULT_STATUS_LDO');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x22,0xFF]); // Write all 1s to LDO FAULT_STATUS register
				    templateObj.logMsgAndNotify("check", "LDO Fault Clear Sent", false, true);
           });


//   Read All Registers Button
             templateObj.$.READ_ALL_REG_BTN.addEventListener("click",function(event){   

				    templateObj.logMsgAndNotify("check", "Reading all registers...", false, true);
				    
				    templateObj.$.ti_widget_register_page.readAllRegistersClickHdlr()
           });


//------------------------------SAFETY BUTTONS----------------------------------------------------//           
//   Clear Safety Error Flags Button
             templateObj.$.SAFETY_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.DEV_ERR_ACK_1');  
			    bind.setValue(0x7F);
				
//				    i2c.I2C_Write(1,0x60,[0x2B,0x7F]); // Write all 1s to DEV_ERR_ACK_1 register
				    templateObj.logMsgAndNotify("check", "Reset Safety Error Flags", false, true);
           });

//   Clear Interface Faults Button
             templateObj.$.INTERFACE_FAULT_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.DEV_ERR_ACK_2');  
			    bind.setValue(0x1F);
				
//				    i2c.I2C_Write(1,0x60,[0x2C,0x1F]); // Write all 1s to DEV_ERR_ACK_2 register
				    templateObj.logMsgAndNotify("check", "Reset Interface Faults", false, true);
           });
           
//   Clear DEV Faults 1 Button
             templateObj.$.DEV_FAULT_1_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.DEV_FAULT_STATUS_1');  
			    bind.setValue(0x3F);
				
//				    i2c.I2C_Write(1,0x60,[0x2D,0x3F]); // Write all 1s to DEV_FAULT_STATUS_1 register
				    templateObj.logMsgAndNotify("check", "Reset Status 1 Flags", false, true);
           });
           
//   Clear DEV Faults 2 Button
             templateObj.$.DEV_FAULT_2_CLEAR_BTN.addEventListener("click",function(event){   
                
			    var bind = gc.databind.registry.getBinding('widget.my_model.DEV_FAULT_STATUS_2');  
			    bind.setValue(0xFF);
				
//				    i2c.I2C_Write(1,0x60,[0x2E,0x1F]); // Write all 1s to DEV_FAULT_STATUS_2 register
				    templateObj.logMsgAndNotify("check", "Reset Status 2 Flags", false, true);
           });
//----------------------------------------------------------------------------------//   

            templateObj.$.UPDATE_TIMING_DIAGRAM_BTN.addEventListener("click", function(event) {
              
                const logicHigh = 1;
                const maxSequence = 800;
                
                var B1_VOUT = parseFloat(templateObj.$.BUCK1_VOUT.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.BUCK1_VOUT').getValue()],10);
                var B2_VOUT = parseFloat(templateObj.$.BUCK2_VOUT.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.BUCK2_VOUT').getValue()],10);
                var B3_VOUT = parseFloat(templateObj.$.BUCK3_VOUT.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.BUCK3_VOUT').getValue()],10);
                var LDO_VOUT = parseFloat(templateObj.$.LDO_VOUT.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.LDO_VOUT').getValue()],10);
                
                var B1_MASKS_PREV = 0;
                var B2_MASKS_PREV = 0;
                var B3_MASKS_PREV = 0;
                var LDO_MASKS_PREV = 0;
                var nRST_MASKS_PREV = 0;
                var GPIO_MASKS_PREV = 0;
                
                var B1_DELAY = gc.databind.registry.getBinding('widget.my_model.BUCK1_SEQ_DLY').getValue();
                var B2_DELAY = gc.databind.registry.getBinding('widget.my_model.BUCK2_SEQ_DLY').getValue();
                var B3_DELAY = gc.databind.registry.getBinding('widget.my_model.BUCK3_SEQ_DLY').getValue();
                var LDO_DELAY = gc.databind.registry.getBinding('widget.my_model.LDO_SEQ_DLY').getValue();
                var nRST_DELAY = gc.databind.registry.getBinding('widget.my_model.nRSTOUT_SEQ_DLY').getValue();
                var GPIO_DELAY = gc.databind.registry.getBinding('widget.my_model.GPIO_SEQ_DLY').getValue();
                
                var B1_MASKS;
                var B2_MASKS;
                var B3_MASKS;
                var LDO_MASKS;
                var nRST_MASKS;
                var GPIO_MASKS;
                
                var B1_ON_DELAY;
                var B2_ON_DELAY;
                var B3_ON_DELAY;
                var LDO_ON_DELAY;
                var nRST_ON_DELAY;
                var GPIO_ON_DELAY;
                
                var PWR_ON_SEQ;
                var SEQ_SEQ;
                var B1_SEQ;
                var B2_SEQ;
                var B3_SEQ;
                var LDO_SEQ;
                var nRST_SEQ;
                var GPIO_SEQ;
                
                var B1_TRIG = 0;
                var B2_TRIG = 0;
                var B3_TRIG = 0;
                var LDO_TRIG = 0;
                var nRST_TRIG = 0;
                var GPIO_TRIG = 0; 
                
                var B1_EN = gc.databind.registry.getBinding('widget.my_model.BUCK_LDO_CTRL.ENABLE_BUCK1').getValue();
                var B2_EN = gc.databind.registry.getBinding('widget.my_model.BUCK_LDO_CTRL.ENABLE_BUCK2').getValue();
                var B3_EN = gc.databind.registry.getBinding('widget.my_model.BUCK_LDO_CTRL.ENABLE_BUCK3').getValue();
                var LDO_EN = gc.databind.registry.getBinding('widget.my_model.BUCK_LDO_CTRL.ENABLE_LDO').getValue();
                
                var B1_RC = 10/1000000*parseInt(templateObj.$.BUCK1_Discharge_Control.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.DISCHARGE_SETTING.BUCK1_Discharge_Control').getValue()],10);
                var B2_RC = 10/1000000*parseInt(templateObj.$.BUCK2_Discharge_Control.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.DISCHARGE_SETTING.BUCK2_Discharge_Control').getValue()],10);
                var B3_RC = 10/1000000*parseInt(templateObj.$.BUCK3_Discharge_Control.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.DISCHARGE_SETTING.BUCK3_Discharge_Control').getValue()],10);
                var LDO_RC = 2/1000000*parseInt(templateObj.$.LDO_Discharge_Control.labels.split("|")[gc.databind.registry.getBinding('widget.my_model.DISCHARGE_SETTING.LDO_Discharge_Control').getValue()],10);
                
                var PWR_DOWN_SEQUENCE = templateObj.$.Sequence_Selection.checked; 
                var SEQ_ON_DELAY = PWR_DOWN_SEQUENCE ? templateObj.$.SEQ_OFF_DELAY.value : templateObj.$.SEQ_ON_DELAY.value;
                
                if (!PWR_DOWN_SEQUENCE) {
                    
                    // Power up sequence
                    
                    B1_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_BUCK1').getValue(); 
                    B2_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_BUCK2').getValue();
                    B3_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_BUCK3').getValue(); 
                    LDO_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_LDO').getValue(); 
                    nRST_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_nRSTOUT').getValue(); 
                    GPIO_MASKS = gc.databind.registry.getBinding('widget.my_model.SEQ_TRIG_GPIO').getValue();
                    
                    B1_ON_DELAY = parseFloat(templateObj.$.BUCK1_SEQ_DLY_ON.labels.split("|")[B1_DELAY & 0x0F],10);
                    B2_ON_DELAY = parseFloat(templateObj.$.BUCK2_SEQ_DLY_ON.labels.split("|")[B2_DELAY & 0x0F],10);
                    B3_ON_DELAY = parseFloat(templateObj.$.BUCK3_SEQ_DLY_ON.labels.split("|")[B3_DELAY & 0x0F],10);
                    LDO_ON_DELAY = parseFloat(templateObj.$.LDO_SEQ_DLY_ON.labels.split("|")[LDO_DELAY & 0x0F],10);
                    nRST_ON_DELAY = parseFloat(templateObj.$.nRSTOUT_DLY_ON.labels.split("|")[nRST_DELAY & 0x0F],10);
                    GPIO_ON_DELAY = parseFloat(templateObj.$.GPIO_DLY_ON.labels.split("|")[GPIO_DELAY & 0x0F],10);
                    
                    PWR_ON_SEQ = [0,0]; //,0]; //,logicDelay,logicHigh]; 
                    SEQ_SEQ = [0,0];
                    B1_SEQ = [0,0,0]; //0,0]; //[0,0,0,0,0,0,0,0,0,0]; 
                    B2_SEQ = [0,0,0]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    B3_SEQ = [0,0,0]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    LDO_SEQ = [0,0,0]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    nRST_SEQ = [0,0,0]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    GPIO_SEQ = [0,0,0];
                    
                }
                
                else {
                    
                    // Power down sequence
                    
                    B1_MASKS = 0x1F;
                    B2_MASKS = 0x1F;
                    B3_MASKS = 0x1F;
                    LDO_MASKS = 0x1F;
                    nRST_MASKS = 0x3F;
                    GPIO_MASKS = 0x3F;
                    
                    B1_ON_DELAY = parseFloat(templateObj.$.BUCK1_SEQ_DLY_OFF.labels.split("|")[(B1_DELAY & 0xF0) >> 4],10);
                    B2_ON_DELAY = parseFloat(templateObj.$.BUCK2_SEQ_DLY_OFF.labels.split("|")[(B2_DELAY & 0xF0) >> 4],10);
                    B3_ON_DELAY = parseFloat(templateObj.$.BUCK3_SEQ_DLY_OFF.labels.split("|")[(B3_DELAY & 0xF0) >> 4],10);
                    LDO_ON_DELAY = parseFloat(templateObj.$.LDO_SEQ_DLY_OFF.labels.split("|")[(LDO_DELAY & 0xF0) >> 4],10);
                    nRST_ON_DELAY = parseFloat(templateObj.$.nRSTOUT_DLY_OFF.labels.split("|")[(nRST_DELAY & 0xF0) >> 4],10);
                    GPIO_ON_DELAY = parseFloat(templateObj.$.GPIO_DLY_OFF.labels.split("|")[(GPIO_DELAY & 0xF0) >> 4],10);             
                    
                    PWR_ON_SEQ = [logicHigh,logicHigh]; //,0]; //,logicDelay,logicHigh];
                    SEQ_SEQ = [logicHigh,logicHigh];

                    nRST_SEQ = [logicHigh,logicHigh,logicHigh]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    GPIO_SEQ = [logicHigh,logicHigh,logicHigh]; 
                    
                    if (B1_EN) B1_SEQ = [B1_VOUT,B1_VOUT,B1_VOUT]; //0,0]; //[0,0,0,0,0,0,0,0,0,0]; 
                    else B1_SEQ = [0,0,0];
                    
                    if (B2_EN) B2_SEQ = [B2_VOUT,B2_VOUT,B2_VOUT]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    else B2_SEQ = [0,0,0];
                    
                    if (B3_EN) B3_SEQ = [B3_VOUT,B3_VOUT,B3_VOUT]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    else B3_SEQ = [0,0,0];
                    
                    if (LDO_EN) LDO_SEQ = [LDO_VOUT,LDO_VOUT,LDO_VOUT]; //,0,0]; //[0,0,0,0,0,0,0,0,0,0];
                    else LDO_SEQ = [0,0,0]; 
                    
                }
                
                var GPIO_as_input = gc.databind.registry.getBinding('widget.my_model.LDO_GPIO_CFG.GPIO_DIR_CONFIG').getValue();
                var GPIO_enabled = gc.databind.registry.getBinding('widget.my_model.GPIO_CTRL.GPIO_STATE').getValue();
                
                var i = 2; 
                
                //while ((B1_MASKS != 0x3F || B2_MASKS != 0x3F || B3_MASKS != 0x3F || LDO_MASKS != 0x3F) && i < 160 ) {
                    
                while (((B1_EN && B1_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : B1_VOUT)) || 
                        (B2_EN && B2_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : B2_VOUT)) || 
                        (B3_EN && B3_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : B3_VOUT)) || 
                        (LDO_EN && LDO_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : LDO_VOUT)) || 
                        nRST_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : logicHigh) ||
                        GPIO_SEQ[i] != (PWR_DOWN_SEQUENCE ? 0 : logicHigh)) && 
                        i < maxSequence) {
                    
                    //PWR_ON_SEQ.push(stdDelay,logicHigh); 
                    
                    PWR_ON_SEQ.push(PWR_DOWN_SEQUENCE ? 0 : logicHigh);
                    
                    if (!PWR_DOWN_SEQUENCE) SEQ_SEQ.push((i - 2 >= 4*SEQ_ON_DELAY) ? logicHigh : 0);
                    else SEQ_SEQ.push((i - 2 >= 4*SEQ_ON_DELAY) ? 0 : logicHigh);
                    
                    if (PWR_ON_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : logicHigh))  {
                        B1_MASKS |= 0x20; 
                        B2_MASKS |= 0x20;
                        B3_MASKS |= 0x20;
                        LDO_MASKS |= 0x20;
                        nRST_MASKS |= 0x40; 
                        GPIO_MASKS |= 0x40;
                    }
                    
                    if (SEQ_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : logicHigh)) {
                        B1_MASKS |= 0x01; 
                        B2_MASKS |= 0x01;
                        B3_MASKS |= 0x01;
                        LDO_MASKS |= 0x01;
                        nRST_MASKS |= 0x01; 
                        GPIO_MASKS |= 0x01;                        
                    }
                    
                    if (GPIO_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : logicHigh)) {
                        B1_MASKS |= 0x02; 
                        B2_MASKS |= 0x02;
                        B3_MASKS |= 0x02;
                        LDO_MASKS |= 0x02;
                        nRST_MASKS |= 0x02; 
                        GPIO_MASKS |= 0x02;                        
                    }
                    
                    if (nRST_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : logicHigh)) GPIO_MASKS |= 0x02;                        
                    
                    if (B1_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : B1_VOUT))  {
                        B2_MASKS |= 0x04;
                        B3_MASKS |= 0x04;
                        LDO_MASKS |= 0x04;
                        nRST_MASKS |= 0x04;
                        GPIO_MASKS |= 0x04;
                    }
                    
                    if (B2_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : B2_VOUT))  {
                        B1_MASKS |= 0x04;
                        B3_MASKS |= 0x08;
                        LDO_MASKS |= 0x08;
                        nRST_MASKS |= 0x08;
                        GPIO_MASKS |= 0x08; 
                    }
                    
                    if (B3_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : B3_VOUT))  {
                        B1_MASKS |= 0x08;
                        B2_MASKS |= 0x08;
                        LDO_MASKS |= 0x10;
                        nRST_MASKS |= 0x10;
                        GPIO_MASKS |= 0x10; 
                    }
                    
                    if (LDO_SEQ[i] == (PWR_DOWN_SEQUENCE ? 0 : LDO_VOUT))  {
                        B1_MASKS |= 0x10;
                        B2_MASKS |= 0x10;
                        B3_MASKS |= 0x10;
                        nRST_MASKS |= 0x20;
                        GPIO_MASKS |= 0x20; 
                    }
                    
                    if (B1_MASKS == 0x3F && B1_MASKS_PREV != 0x3F) { B1_TRIG = i; }
                    if (B2_MASKS == 0x3F && B2_MASKS_PREV != 0x3F) { B2_TRIG = i; }
                    if (B3_MASKS == 0x3F && B3_MASKS_PREV != 0x3F) { B3_TRIG = i; }
                    if (LDO_MASKS == 0x3F && LDO_MASKS_PREV != 0x3F) { LDO_TRIG = i; }
                    if (nRST_MASKS == 0x7F && nRST_MASKS_PREV != 0x7F) { nRST_TRIG = i; }
                    if (GPIO_MASKS == 0x7F && GPIO_MASKS_PREV != 0x7F) { GPIO_TRIG = i; }
                    
                    if (!PWR_DOWN_SEQUENCE) {
                        
                        if (!B1_EN) B1_SEQ.push(0); 
                        else {
                            if (B1_TRIG) 
                                if      (i - B1_TRIG > 4*B1_ON_DELAY) B1_SEQ.push(B1_VOUT);
                                else if (i - B1_TRIG == 4*B1_ON_DELAY) B1_SEQ.push(B1_VOUT/2);
                                else    B1_SEQ.push(0); 
                            else B1_SEQ.push(0); 
                        }
                        
                        if (!B2_EN) B2_SEQ.push(0); 
                        else {
                            if (B2_TRIG) 
                                if      (i - B2_TRIG > 4*B2_ON_DELAY) B2_SEQ.push(B2_VOUT);
                                else if (i - B2_TRIG == 4*B2_ON_DELAY) B2_SEQ.push(B2_VOUT/2);
                                else    B2_SEQ.push(0); 
                            else B2_SEQ.push(0); 
                        }
                        
                        if (!B3_EN) B3_SEQ.push(0);     
                        else {
                            if (B3_TRIG) 
                                if      (i - B3_TRIG > 4*B3_ON_DELAY) B3_SEQ.push(B3_VOUT);
                                else if (i - B3_TRIG == 4*B3_ON_DELAY) B3_SEQ.push(B3_VOUT/2);
                                else    B3_SEQ.push(0); 
                            else B3_SEQ.push(0); 
                        }
                        
                        if (!LDO_EN) LDO_SEQ.push(0); 
                        else LDO_SEQ.push((LDO_TRIG && (i - LDO_TRIG) >= 4*LDO_ON_DELAY) ? LDO_VOUT : 0);
                        
                        nRST_SEQ.push((nRST_TRIG && (i - nRST_TRIG) >= 4*nRST_ON_DELAY) ? logicHigh : 0);
                        GPIO_SEQ.push((GPIO_TRIG && (i - GPIO_TRIG) >= 4*GPIO_ON_DELAY) ? logicHigh : 0);
                        
                    } else {
                        
                        var push; 
                        
                        if (!B1_EN) B1_SEQ.push(0);
                        else {
                            if (B1_TRIG)
                                if (i - B1_TRIG >= 4*B1_ON_DELAY) {
                                    push = B1_VOUT*Math.exp(-1*(i-1-4*B1_ON_DELAY)/(4*1000*B1_RC));
                                    B1_SEQ.push(push < 0.01 ? 0 : push); 
                                } else B1_SEQ.push(B1_VOUT); 
                            else B1_SEQ.push(B1_VOUT);
                        }
                        
                        if (!B2_EN) B2_SEQ.push(0);   
                        else {
                            if (B2_TRIG)
                                if (i - B2_TRIG >= 4*B2_ON_DELAY) {
                                    push = B2_VOUT*Math.exp(-1*(i-1-4*B2_ON_DELAY)/(4*1000*B2_RC));
                                    B2_SEQ.push(push < 0.01 ? 0 : push); 
                                } else B2_SEQ.push(B2_VOUT); 
                            else B2_SEQ.push(B2_VOUT);
                        }
                        
                        if (!B3_EN) B3_SEQ.push(0);
                        else {
                            if (B3_TRIG)
                                if  (i - B3_TRIG >= 4*B3_ON_DELAY) { 
                                    push = B3_VOUT*Math.exp(-1*(i-1-4*B3_ON_DELAY)/(4*1000*B3_RC));
                                    B3_SEQ.push(push < 0.01 ? 0 : push); 
                                } else B3_SEQ.push(B3_VOUT); 
                            else B3_SEQ.push(B3_VOUT);
                        }
                        
                        if (!LDO_EN) LDO_SEQ.push(0); 
                        else {
                            if (LDO_TRIG)
                                if  (i - LDO_TRIG >= 4*LDO_ON_DELAY) {
                                    push = LDO_VOUT*Math.exp(-1*(i-1-4*LDO_ON_DELAY)/(4*1000*LDO_RC));
                                    LDO_SEQ.push(push < 0.01 ? 0 : push); 
                                } else LDO_SEQ.push(LDO_VOUT); 
                            else LDO_SEQ.push(LDO_VOUT);    
                        }
                        
                        //LDO_SEQ.push((LDO_TRIG && (i - LDO_TRIG) >= 4*LDO_ON_DELAY) ? 0 : LDO_VOUT);
                        
                        nRST_SEQ.push((nRST_TRIG && (i - nRST_TRIG) >= 4*nRST_ON_DELAY) ? 0 : logicHigh);
                        GPIO_SEQ.push((GPIO_TRIG && (i - GPIO_TRIG) >= 4*GPIO_ON_DELAY) ? 0 : logicHigh);
                    }
                    
                    B1_MASKS_PREV = B1_MASKS;
                    B2_MASKS_PREV = B2_MASKS;
                    B3_MASKS_PREV = B3_MASKS;
                    LDO_MASKS_PREV = LDO_MASKS;
                    nRST_MASKS_PREV = nRST_MASKS; 
                    GPIO_MASKS_PREV = GPIO_MASKS; 
                    i++;
                }

                B1_SEQ.push(B1_SEQ[B1_SEQ.length-1]);
                B2_SEQ.push(B2_SEQ[B2_SEQ.length-1]);
                B3_SEQ.push(B3_SEQ[B3_SEQ.length-1]);
                LDO_SEQ.push(LDO_SEQ[LDO_SEQ.length-1]);
                PWR_ON_SEQ.push(PWR_ON_SEQ[PWR_ON_SEQ.length-1]);
                SEQ_SEQ.push(SEQ_SEQ[SEQ_SEQ.length-1]);
                nRST_SEQ.push(nRST_SEQ[nRST_SEQ.length-1]);
                GPIO_SEQ.push(GPIO_SEQ[GPIO_SEQ.length-1]);
                
                if (i == maxSequence && !PWR_DOWN_SEQUENCE) templateObj.logMsgAndNotify("error", "Invalid Sequence Settings", false, true);
                
                else {
                
                    var B1_series = templateObj.$.ti_widget_linegraph_regulators.genXSeries(B1_SEQ,0,0.25);
                    var B2_series = templateObj.$.ti_widget_linegraph_regulators.genXSeries(B2_SEQ,0,0.25);
                    var B3_series = templateObj.$.ti_widget_linegraph_regulators.genXSeries(B3_SEQ,0,0.25);
                    var LDO_series = templateObj.$.ti_widget_linegraph_regulators.genXSeries(LDO_SEQ,0,0.25);
                    var PWR_ON_series = templateObj.$.ti_widget_linegraph_logic.genXSeries(PWR_ON_SEQ,0,0.25);
                    var SEQ_series = templateObj.$.ti_widget_linegraph_logic.genXSeries(SEQ_SEQ,0,0.25);
                    var nRST_series = templateObj.$.ti_widget_linegraph_logic.genXSeries(nRST_SEQ,0,0.25);
                    
                    templateObj.$.ti_widget_linegraph_regulators.series_0_min = 0;
                    templateObj.$.ti_widget_linegraph_regulators.series_1_min = 0;
                    templateObj.$.ti_widget_linegraph_regulators.series_2_min = 0;
                    templateObj.$.ti_widget_linegraph_regulators.series_3_min = 0;                    
                    templateObj.$.ti_widget_linegraph_regulators.series_0_max = 4;
                    templateObj.$.ti_widget_linegraph_regulators.series_1_max = 4;
                    templateObj.$.ti_widget_linegraph_regulators.series_2_max = 4;
                    templateObj.$.ti_widget_linegraph_regulators.series_3_max = 4;
                    
                    templateObj.$.ti_widget_linegraph_regulators.series_1_values = B2_series.join(';');
                    templateObj.$.ti_widget_linegraph_regulators.series_2_values = B3_series.join(';');
                    templateObj.$.ti_widget_linegraph_regulators.series_3_values = LDO_series.join(';');                    
                    
                    templateObj.$.ti_widget_linegraph_regulators.series_0_values = B1_series.join(';');
                    templateObj.$.ti_widget_linegraph_regulators.series_1_values = B2_series.join(';');
                    templateObj.$.ti_widget_linegraph_regulators.series_2_values = B3_series.join(';');
                    templateObj.$.ti_widget_linegraph_regulators.series_3_values = LDO_series.join(';');
                    
                    templateObj.$.ti_widget_linegraph_logic.series_0_values = PWR_ON_series.join(';');
                    templateObj.$.ti_widget_linegraph_logic.series_1_values = SEQ_series.join(';');
                    templateObj.$.ti_widget_linegraph_logic.series_3_values = nRST_series.join(';');
                    
                    if (!GPIO_as_input && GPIO_enabled) {
                        var GPIO_series = templateObj.$.ti_widget_linegraph_logic.genXSeries(GPIO_SEQ,0,0.25);
                        templateObj.$.ti_widget_linegraph_logic.series_2_values = GPIO_series.join(';');
                    }
                    
                    else templateObj.$.ti_widget_linegraph_logic.series_2_values = 0; 
                
                }
                
            });

//---------------------------------------------------------------------------------//

            templateObj.$.WD_ERROR_CLEAR_BTN.addEventListener("click", function(event) {
              
                var bind = gc.databind.registry.getBinding('widget.my_model.WD_ERR_STATUS');
                bind.setValue(0x1F);
              
                templateObj.logMsgAndNotify("check", "Reset Watchdog Faults", false, true);
                
            });

            templateObj.$.WD_WIN1_SLIDER.addEventListener("changed", function(event) {
                
                var bind = gc.databind.registry.getBinding('widget.my_model.WD_WIN1_CFG');
                
                var label = (((bind.getValue() + 1) * 0.55).toPrecision(3)).toString();
                templateObj.$.WD_WIN1.label = label.concat(' ms');                 
            
            });
            
            templateObj.$.WD_WIN2_SLIDER.addEventListener("changed", function(event) {
                
                var bind = gc.databind.registry.getBinding('widget.my_model.WD_WIN2_CFG');
                
                var label = (((bind.getValue() + 1) * 0.55).toPrecision(3)).toString();
                templateObj.$.WD_WIN2.label = label.concat(' ms');                 
            
            });
            
            templateObj.$.WD_LONGWIN_SLIDER.addEventListener("changed", function(event) {
                
                var bind = gc.databind.registry.getBinding('widget.my_model.WD_LONGWIN_CFG');
                
                var label = (((bind.getValue() * 3) + 0.1).toPrecision(3)).toString();
                templateObj.$.WD_LONGWIN.label = label.concat(' s');                 
            
            });    

            /*templateObj.$.CONTROLLER_SELECTION.addEventListener("selected", function(event) {
                
                var bind = gc.databind.registry.getBinding('my_model.$unit');
                
                bind.setValue(parseInt(templateObj.$.CONTROLLER_SELECTION.selectedValue));
                
            })*/

//----------------------------------------------------------------------------------// 

           registerModel.addEventListener('registerDefsChanged', function(event) {
            
                registerModel.getRegisterViewInformation().then(function(deviceObj) {
                   
                    const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.name === "UserRegisters-user_mem"));
                    
                    if (regBlockArray.length > 0) {
                        
                        const watchdogWindowRegs = regBlockArray[0].registers.filter((reg) => (reg.addr > 0x38 && reg.addr < 0x3C));   
                        
                        if (watchdogWindowRegs.length > 0) {
                            
                            var win1_label = ((parseInt(watchdogWindowRegs[0].value, 16) + 1)*0.55).toString();
                            var win2_label = ((parseInt(watchdogWindowRegs[1].value, 16) + 1)*0.55).toString();
                            var longwin_label = (parseInt(watchdogWindowRegs[2].value, 16) * 3 + 0.1).toString();
                            
                            templateObj.$.WD_WIN1.label = win1_label.concat(' ms');
                            templateObj.$.WD_WIN2.label = win2_label.concat(' ms');
                            templateObj.$.WD_LONGWIN.label = longwin_label.concat(' s');
                        }
                        
                    }
                    
                });
               
           });
           
           
        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init);
}

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/

    /*gc.databind.registry.getBinding('my_model.WD_WIN1_CFG').addChangedListener({
        onValueChanged(oldValue, newValue) {
         
                var readOnly = gc.databind.registry.getBinding('my_model.WD_WIN1_CFG').excludeFromStorageProviderData;
                
                var label = (((newValue + 1) * 0.55).toPrecision(3)).toString();
                templateObj.$.WD_WIN1.label = label.concat(' ms');    
        
        }
    });
    
    gc.databind.registry.getBinding('my_model.WD_WIN2_CFG').addChangedListener({
        onValueChanged(oldValue, newValue) {
         
                var readOnly = gc.databind.registry.getBinding('my_model.WD_WIN2_CFG').excludeFromStorageProviderData;
         
                var label = (((newValue + 1) * 0.55).toPrecision(3)).toString();
                templateObj.$.WD_WIN2.label = label.concat(' ms');    
            
        }
    });
    
    gc.databind.registry.getBinding('my_model.WD_LONGWIN_CFG').addChangedListener({
        onValueChanged(oldValue, newValue) {
        
                var readOnly = gc.databind.registry.getBinding('my_model.WD_LONGWIN_CFG').excludeFromStorageProviderData;
        
                var label = (newValue *3 + 0.1).toPrecision(3).toString();
                templateObj.$.WD_LONGWIN.label = label.concat(' s');    
            
        }
    }); */

		//if (templateObj.I2C_CRC_Enabled) {
			gc.databind.AEVMCodec.prototype.register_crc_user({
				embed_crc_data: function(crc, arg) {
					if (templateObj.I2C_CRC_Enabled && arg.write) {
						// TODO Gerard, refer arg.write, arg.deviceAddr, arg.opcode, arg.payload, arg.num_read_bytes
						// and use crc.checksum(....) to modify payload, and modify num_read_bytes
						// console.log(arg); // for debug only
						// return {payload: ......, num_read_bytes: .... }
						var c = crc.checksum([arg.deviceAddr<<1|(!arg.write)], 0xff);
						c = crc.checksum([arg.opcode], c);
                        c = crc.checksum([arg.payload], c); 
                        arg.payload.push(c);
                        arg.num_read_bytes += 2; 
                        return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes};
					} else {
						return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes};
					}
				}
				, verify_crc_data: function(crc, arg) {
					console.log(arg); // for debug only
					if (templateObj.I2C_CRC_Enabled) {
						// TODO Gerard, refer arg.payload and use crc to check the bytes in arg.payload
						// console.log(arg); // for debug only
						// and return {valid: true or false, payload: payload without crc bytes};
						return {valid: true, payload: arg.payload};
					} else {
						return {valid: true, payload: arg.payload};
					}
				}
			}, 'i2c');
		//}
        
        
        //if (templateObj.I2C_CRC_Enabled) {
            gc.databind.USB2ANY.prototype.register_crc_user( {
            // embed_crc_data is called back during read and write for inserting crc bytes
            //   crc is provided to compute crc byte(s)
            //   arg: {write, deviceAddr, payload}
            //   arg.write indicates this callback is for write (true) or read(false)
            //   arg.deviceAddr is the device address (a.k.a. slave address)
            //   arg.payload is the payload part of (u2a/aevm) packet.
            embed_crc_data: function(crc, arg) {
                // use crc.checksum(byte_array, optionalPreviousCRC) to compute crc data byte(s)
                // and prepend/insert/append crc data byte(s) into payload at desired place(s)
                
                // When writing this code, you can use console.log(arg) to see what properties are in 'arg'.
    
                // Case of AEVM: arg.num_read_bytes is the number of bytes to read without counting the crc part,
                // modify it to include the number of crc bytes for read.
                // Example code for AEVM
                // if (arg.write === true) {
                //     var c = crc.checksum([arg.deviceAddr]);
                //     if (arg.opcode !== undefined) c = crc.checksum([arg.opcode],c);
                //     c = crc.checksum(arg.payload, c);
                //     arg.payload.push(c);
                // }
                // return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes+1};
    
                // Case of USB2ANY: arg.numBytes is number of bytes to read/write without crc part
                // return modified payload and number of bytes (data and crc parts)
                // Example code (uncommnet the code below)
                var extra = 0;
                var workAroundBug = (arg.write===true) ? 1 : 0;
                // if (arg.write === true) {
                //    arg.payload.push(crc.checksum(arg.payload));
                //}
                // extra = 1; // number of crc bytes added to arg.payload for write
                           // number of crc bytes for read
                //return {payload: arg.payload, numBytes: arg.numBytes+extra + workAroundBug }
                
                if (templateObj.I2C_CRC_Enabled && arg.write) {
                    var c = crc.checksum([arg.deviceAddr<<1|(!arg.write)], 0xff);
                    c = crc.checksum([arg.registerAddr], c);
                    c = crc.checksum([arg.writeData], c); 
                    arg.payload.push(c);
                    arg.numBytes+=2;
            	    return {payload: arg.payload, numBytes: arg.numBytes+extra + workAroundBug }
                }
    
            }
            // verify_crc_data is called back after getting result of read request from controller for
            // verifying crc byte(s) and extracting actual data part out from payload
            //   crc is provided to compute crc byte(s)
            //   arg: {payload}
            //   arg.payload is the payload including both crc and data parts
            , verify_crc_data: function(crc, arg) {
                // Example code (uncommnet the code below)
                var d = arg.payload, valid = true;
                //    d = arg.payload.slice(0,-1); // the data without crc part
                //    c = crc.checksum(d);
                //    valid = c == arg.payload[arg.payload.length-1];
                // If you do not plan to verify the crc byte(s),
                // you still need to say valid is true, and make sure d is the
                // data without crc part. Otherwise, your app cannot properly read registers
                // and may get "connected to controller but failed to connect to device...".
                return {valid: valid, payload: d}
            }
            }, 'i2c');
        //}
	
        /*if (templateObj.I2C_CRC_Enabled) {
            gc.databind.USB2ANY.prototype.register_crc_user({
                embed_crc_data: function(crc, arg) {
                    
                    // i2c crc calculation and transmission
                    if (templateObj.I2C_CRC_Enabled && arg.write) {
                        var c = crc.checksum([arg.deviceAddr<<1|(!arg.write)], 0xff);
                        c = crc.checksum([arg.registerAddr], c);
                        c = crc.checksum([arg.writeData], c); 
                        arg.payload.push(c);
                        arg.numBytes+=2;
        	            return {payload: arg.payload, numBytes: arg.numBytes};
                    }
                }, 
                verify_crc_data: function(crc, arg) {
                    //Not verifying anything for now
                    var d = arg.payload;
                    var valid = true;
                    if (templateObj.I2C_CRC_Enabled) {
                        d = arg.payload.slice(0,-1);
                        // valid = ….
                    }
                    return {valid: valid, payload: d}
                }
            }, 'i2c');
            
            gc.databind.AEVMCodec.prototype.register_crc_user({
                embed_crc_data: function(crc, arg) {
                    if (templateObj.I2C_CRC_Enabled && arg.write) {
                        // TODO Gerard, refer arg.write, arg.deviceAddr, arg.opcode, arg.payload, arg.num_read_bytes
                        // and use crc.checksum(....) to modify payload, and modify num_read_bytes
                        // console.log(arg); // for debug only
                        // return {payload: ......, num_read_bytes: .... }
                        var c = crc.checksum([arg.deviceAddr<<1|(!arg.write)], 0xff);
                        c = crc.checksum([arg.opcode], c);
                        c = crc.checksum([arg.payload], c); 
                        arg.payload.push(c);
                        arg.num_read_bytes+=2;
                        return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes};
                    } else {
                        return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes};
                    }
                }
            , verify_crc_data: function(crc, arg) {
                console.log(arg); // for debug only
                if (templateObj.I2C_CRC_Enabled) {
                    // TODO Gerard, refer arg.payload and use crc to check the bytes in arg.payload
                    // console.log(arg); // for debug only
                    // and return {valid: true or false, payload: payload without crc bytes};
                    return {valid: true, payload: arg.payload};
                } else {
                    return {valid: true, payload: arg.payload};
                }
            }
            }, 'i2c');
        }*/

});